@@ -14,8 +14,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import socket,struct
-
 import time
 from IM.uriparse import uriparse
 from IM.VirtualMachine import VirtualMachine
@@ -25,7 +23,7 @@ from libcloud.compute.base import NodeImage, NodeAuthSSHKey
 from libcloud.compute.types import NodeState, Provider
 from libcloud.compute.providers import get_driver
 
-from IM.radl.radl import network, Feature
+from IM.radl.radl import Feature
 
 class LibCloudCloudConnector(CloudConnector):
 	"""
@@ -80,21 +78,32 @@ class LibCloudCloudConnector(CloudConnector):
 	
 	def get_instance_type(self, sizes, radl):
 		"""
-		Get the name of the instance type to launch to EC2
+		Get the name of the instance type to launch to LibCloud
 
 		Arguments:
 		   - size(list of :py:class: `libcloud.compute.base.NodeSize`): List of sizes on a provider
 		   - radl(str): RADL document with the requirements of the VM to get the instance type
 		Returns: a :py:class:`libcloud.compute.base.NodeSize` with the instance type to launch	
 		"""
+		instance_type_name = radl.getValue('instance_type')
+		
 		memory = radl.getFeature('memory.size').getValue('M')
+		memory_op = radl.getFeature('memory.size').getLogOperator()
+		disk_free = 0
+		disk_free_op = ">="
+		if radl.getValue('disk.0.free_size'):
+			disk_free = radl.getFeature('disk.0.free_size').getValue('G')
+			disk_free_op = radl.getFeature('memory.size').getLogOperator()
 
 		res = None
 		for size in sizes:
-			# get the node size with the lowest price
-			if res is None or (size.price <= res.price):
-				if size.ram >= memory:
-					res = size
+			# get the node size with the lowest price and memory (in the case of the price is not set)
+			if res is None or (size.price <= res.price and size.ram <= res.ram):
+				str_compare = "size.ram " + memory_op + " memory"
+				str_compare += " and size.disk " + disk_free_op + " disk_free"
+				if eval(str_compare):
+					if not instance_type_name or size.name == instance_type_name:
+						res = size
 		
 		if res is None:
 			self.logger.error("No compatible size found")
@@ -118,8 +127,18 @@ class LibCloudCloudConnector(CloudConnector):
 				instance_type = self.get_instance_type(driver.list_sizes(), res_system)
 				
 				res_system.addFeature(Feature("memory.size", "=", instance_type.ram, 'M'), conflict="other", missing="other")
-				res_system.addFeature(Feature("disk.0.free_size", "=", instance_type.disk , 'G'), conflict="other", missing="other")
-				res_system.addFeature(Feature("price", "=", instance_type.price), conflict="me", missing="other")
+				if instance_type.disk:
+					res_system.addFeature(Feature("disk.0.free_size", "=", instance_type.disk , 'G'), conflict="other", missing="other")
+				if instance_type.price:
+					res_system.addFeature(Feature("price", "=", instance_type.price), conflict="me", missing="other")
+				
+				res_system.addFeature(Feature("instance_type", "=", instance_type.name), conflict="other", missing="other")
+				
+				res_system.addFeature(Feature("provider.type", "=", self.type), conflict="other", missing="other")
+				if self.cloud.server:
+					res_system.addFeature(Feature("provider.host", "=", self.cloud.server), conflict="other", missing="other")
+				if self.cloud.port != -1:
+					res_system.addFeature(Feature("provider.port", "=", self.cloud.port), conflict="other", missing="other")				
 					
 				return [res_system]
 		else:
@@ -135,7 +154,19 @@ class LibCloudCloudConnector(CloudConnector):
 		"""
 		return uriparse(path)[2][1:]
 
-	def launch(self, radl, requested_radl, num_vm, auth_data):
+	@staticmethod
+	def driver_uses_keypair(driver):
+		#return "ssh_key" in driver.features.get("create_node", [])
+		try:
+			driver.get_key_pair("keypair")
+		except NotImplementedError:
+			return False
+		except Exception:
+			return True
+		else:
+			return True
+
+	def launch(self, inf, radl, requested_radl, num_vm, auth_data):
 		driver = self.get_driver(auth_data)
 
 		system = radl.systems[0]
@@ -154,20 +185,22 @@ class LibCloudCloudConnector(CloudConnector):
 		
 		keypair = None
 		public_key = system.getValue("disk.0.os.credentials.public_key")
-		if "ssh_key" in driver.features.get("create_node", []):
-			
+		if self.driver_uses_keypair(driver):			
 			if public_key:
 				keypair = driver.get_key_pair(public_key)
 				if keypair:
 					system.setUserKeyCredentials(system.getCredentials().username, None, keypair.private_key)
 				else:
-					args["auth"] = NodeAuthSSHKey(public_key)
+					if "ssh_key" in driver.features.get("create_node", []):
+						args["auth"] = NodeAuthSSHKey(public_key)
+					else:
+						args["ex_keyname"] = keypair.name
 			elif not system.getValue("disk.0.os.credentials.password"):
-				keypair_name = "im-%s" % time.time()
+				keypair_name = "im-%d" % int(time.time()*100.0)
 				keypair = driver.create_key_pair(keypair_name)
 				system.setUserKeyCredentials(system.getCredentials().username, None, keypair.private_key)
 				
-				if keypair.public_key:
+				if keypair.public_key and "ssh_key" in driver.features.get("create_node", []):
 					args["auth"] = NodeAuthSSHKey(keypair.public_key)
 				else:
 					args["ex_keyname"] = keypair_name
@@ -180,8 +213,8 @@ class LibCloudCloudConnector(CloudConnector):
 			node = driver.create_node(**args)
 			
 			if node:
-				vm = VirtualMachine(node.id, self.cloud, radl, requested_radl)
-				# Add the keypair name to remove it later 
+				vm = VirtualMachine(inf, node.id, self.cloud, radl, requested_radl)
+				# Add the keypair name to remove it later
 				vm.keypair = keypair
 				self.logger.debug("Node successfully created.")
 				res.append((True, vm))
@@ -192,12 +225,12 @@ class LibCloudCloudConnector(CloudConnector):
 
 		return res
 		
-	def get_node_with_id(self, id, auth_data):
+	def get_node_with_id(self, node_id, auth_data):
 		"""
 		Get the node with the specified ID
 
 		Arguments:
-		   - id(str): ID of the node to get
+		   - node_id(str): ID of the node to get
 		   - auth(Authentication): parsed authentication tokens.
 		Returns: a :py:class:`libcloud.compute.base.Node` with the node info	
 		"""
@@ -206,7 +239,7 @@ class LibCloudCloudConnector(CloudConnector):
 		
 		res = None
 		for node in nodes:
-			if node.id == id:
+			if node.id == node_id:
 				res = node
 		return res
 		
@@ -217,7 +250,7 @@ class LibCloudCloudConnector(CloudConnector):
 			success = node.destroy()
 			
 			public_key = vm.getRequestedSystem().getValue('disk.0.os.credentials.public_key')
-			if public_key is None or len(public_key) == 0 or (len(public_key) >= 1 and public_key.find('-----BEGIN CERTIFICATE-----') != -1):
+			if vm.keypair and public_key is None or len(public_key) == 0 or (len(public_key) >= 1 and public_key.find('-----BEGIN CERTIFICATE-----') != -1):
 				# only delete in case of the user do not specify the keypair name
 				node.driver.delete_key_pair(vm.keypair)
 			
@@ -269,77 +302,7 @@ class LibCloudCloudConnector(CloudConnector):
 		   - node(:py:class:`libcloud.compute.base.Node`): object to connect to EC2 instance.
 		"""
 
-		now = str(int(time.time()*100))
-		vm_system = vm.info.systems[0]
-
-		if node.public_ips and not set(node.public_ips).issubset(set(node.private_ips)):
-			public_net = None
-			for net in vm.info.networks:
-				if net.isPublic():
-					public_net = net
-					
-			if public_net is None:
-				public_net = network.createNetwork("public." + now, True)
-				vm.info.networks.append(public_net)
-				num_net = vm.getNumNetworkIfaces()
-			else:
-				# If there are are public net, get the ID
-				num_net = vm.getNumNetworkWithConnection(public_net.id)
-				if num_net is None:
-					# There are a public net but it has not been used in this VM
-					num_net = vm.getNumNetworkIfaces()
-
-			for public_ip in node.public_ips:
-				if public_ip not in node.private_ips:
-					vm_system.setValue('net_interface.' + str(num_net) + '.ip', str(public_ip))
-					vm_system.setValue('net_interface.' + str(num_net) + '.connection',public_net.id)
-
-		if node.private_ips:
-			private_net_masks = ["10.0.0.0/8","172.16.0.0/12","192.168.0.0/16","169.254.0.0/16"]
-			private_net_map = {}
-			
-			for private_ip in node.private_ips:
-				private_net_mask = None
-
-				# Get the private network mask
-				for mask in private_net_masks:
-					if self.addressInNetwork(private_ip,mask):
-						private_net_mask = mask
-						break
-				
-				# This must not happen
-				if private_net_mask is None:
-					self.logger.warn("Private IP " + str(private_ip) + " is not a correct private IPV4 address.")
-					break
-				
-				# Search in previous user private ips
-				private_net = None
-				for net_mask, net in private_net_map.iteritems():
-					if self.addressInNetwork(private_ip, net_mask): 	
-						private_net = net								
-
-				# Search in the RADL nets
-				if private_net is None:
-					for net in vm.info.networks:
-						if not net.isPublic() and net not in private_net_map.values():
-							private_net = net
-							private_net_map[private_net_mask] = net
-			
-				# if it is still None, then create a new one
-				if private_net is None:
-					private_net = network.createNetwork("private." + private_net_mask)
-					vm.info.networks.append(private_net)
-					num_net = vm.getNumNetworkIfaces()
-				else:
-					# If there are are private net, get the ID
-					num_net = vm.getNumNetworkWithConnection(private_net.id)
-					if num_net is None:
-						# There are a private net but it has not been used in this VM
-						num_net = vm.getNumNetworkIfaces()
-	
-				vm_system.setValue('net_interface.' + str(num_net) + '.ip', str(private_ip))
-				vm_system.setValue('net_interface.' + str(num_net) + '.connection',private_net.id)
-
+		vm.setIps(node.public_ips, node.private_ips)
 		self.manage_elastic_ips(vm, node)
 		
 	def manage_elastic_ips(self, vm, node):
@@ -395,13 +358,17 @@ class LibCloudCloudConnector(CloudConnector):
 					node.driver.ex_associate_address_with_node(node, elastic_ip)
 					return elastic_ip
 				elif node.driver.name == "OpenStack":
-					pool = node.driver.ex_list_floating_ip_pools()[0]
-					if fixed_ip:
-						floating_ip = node.driver.ex_get_floating_ip(fixed_ip)
+					if node.driver.ex_list_floating_ip_pools():
+						pool = node.driver.ex_list_floating_ip_pools()[0]
+						if fixed_ip:
+							floating_ip = node.driver.ex_get_floating_ip(fixed_ip)
+						else:
+							floating_ip = pool.create_floating_ip()
+						node.driver.ex_attach_floating_ip_to_node(node, floating_ip)
+						return floating_ip
 					else:
-						floating_ip = pool.create_floating_ip()
-					node.driver.ex_attach_floating_ip_to_node(node, floating_ip)
-					return floating_ip
+						self.logger.error("Error adding a Floating IP: No pools available.")
+						return None
 				else:
 					return None
 			except Exception:
@@ -491,23 +458,27 @@ class LibCloudCloudConnector(CloudConnector):
 		Wait a volume (with the state extra parameter) to be in certain state.
 
 		Arguments:
-		   - volume(:py:class:`libcloud.compute.base.StorageVolume`): volume object.
+		   - volume(:py:class:`libcloud.compute.base.StorageVolume`): volume object or boolean.
 		   - state(str): State to wait for (default value 'available').	
 		   - timeout(int): Max time to wait in seconds (default value 60).
 		"""
-		if 'state' in volume.extra:
-			cont = 0
-			while volume.extra['state'] != state and cont < timeout:
-				cont += 2
-				time.sleep(2)
-				for vol in volume.driver.list_volumes():
-					if vol.id == volume.id:
-						volume = vol
-						break
-			return volume.extra['state'] == state
-
-		return True
-		
+		if volume:
+			if 'state' in volume.extra:
+				cont = 0
+				err_states = ["error"]
+				while volume.extra['state'] != state and volume.extra['state'] not in err_states and cont < timeout:
+					cont += 2
+					time.sleep(2)
+					for vol in volume.driver.list_volumes():
+						if vol.id == volume.id:
+							volume = vol
+							break				
+				return volume.extra['state'] == state
+	
+			return True
+		else:
+			return False
+	
 	def attach_volumes(self, vm, node):
 		"""
 		Attach a the required volumes (in the RADL) to the launched node
@@ -524,15 +495,20 @@ class LibCloudCloudConnector(CloudConnector):
 					disk_size = vm.info.systems[0].getFeature("disk." + str(cont) + ".size").getValue('G')
 					disk_device = vm.info.systems[0].getValue("disk." + str(cont) + ".device")
 					self.logger.debug("Creating a %d GB volume for the disk %d" % (int(disk_size), cont))
-					volume_name = "im-%s" % time.time()
+					volume_name = "im-%d" % int(time.time()*100.0)
+					
 					location = self.get_node_location(node)
 					volume = node.driver.create_volume(int(disk_size), volume_name, location = location)
 					success = self.wait_volume(volume)
-					if not success:
-						self.logger.error("Error waiting the volume ID " + str(volume.id))
-					vm.volumes.append(volume)					
-					self.logger.debug("Attach the volume ID " + str(volume.id))
-					volume.attach(node, "/dev/" + disk_device)
+					if success:
+						# Add the volume to the VM to remove it later
+						vm.volumes.append(volume)
+						self.logger.debug("Attach the volume ID " + str(volume.id))
+						volume.attach(node, "/dev/" + disk_device)
+					else:
+						self.logger.error("Error waiting the volume ID " + str(volume.id) + " not attaching to the VM and destroying it.")
+						volume.destroy()
+					
 					cont += 1
 			return True
 		except Exception:
@@ -579,17 +555,4 @@ class LibCloudCloudConnector(CloudConnector):
 
 				if not success:
 					all_ok = False
-		return all_ok
-			
-	@staticmethod
-	def addressInNetwork(ip,net):
-			"""Is an address in a network (format: 10.0.0.0/24)"""
-			ipaddr = struct.unpack('>L',socket.inet_aton(ip))[0]
-			netaddr,bits = net.split('/')
-			netmask = struct.unpack('>L',socket.inet_aton(netaddr))[0]
-			ipaddr_masked = ipaddr & (4294967295<<(32-int(bits)))   # Logical AND of IP address and mask will equal the network address if it matches
-			if netmask == netmask & (4294967295<<(32-int(bits))):   # Validate network address is valid for mask
-				return ipaddr_masked == netmask
-			else:
-				# print "***WARNING*** Network",netaddr,"not valid with mask /"+bits
-				return False
\ No newline at end of file
+		return all_ok
\ No newline at end of file
